@@ -1,8 +1,8 @@
 use crate::parser::{parse, ArgData, ArgKind, Event, EventData, Position};
+use crate::utils::*;
 use crate::Result;
 use anyhow::bail;
 use clap::{Arg, ArgMatches, Command};
-use convert_case::{Case, Casing};
 use std::collections::HashMap;
 use std::ops::Deref;
 
@@ -140,7 +140,7 @@ impl<'a> Cmd<'a> {
                             )
                         }
                         rootdata.names.insert(name, *position);
-                        cmd.name = Some((name, name.to_case(Case::Kebab)));
+                        cmd.name = Some((name, to_kebab_case(name)));
                         rootcmd.cmds.push(cmd);
                     } else if *name == ENTRYPOINT {
                         rootdata.main = true;
@@ -186,7 +186,9 @@ impl<'a> Cmd<'a> {
         let mut values = vec![];
         for arg_data in &self.args {
             if let Some(value) = arg_data.retrieve_match_value(matches) {
-                values.push(value);
+                if !value.is_empty() {
+                    values.push(value);
+                }
             }
         }
         let mut call_fn: Option<String> = None;
@@ -194,7 +196,10 @@ impl<'a> Cmd<'a> {
             if let Some((fn_name, cmd_name)) = &subcmd.name {
                 if let Some((match_name, subcmd_matches)) = matches.subcommand() {
                     if cmd_name.as_str() == match_name {
-                        values.push(subcmd.retrieve(subcmd_matches, runner));
+                        let value = subcmd.retrieve(subcmd_matches, runner);
+                        if !value.is_empty() {
+                            values.push(value);
+                        }
                         call_fn = Some(fn_name.to_string());
                     }
                 }
@@ -216,7 +221,7 @@ impl<'a> Cmd<'a> {
                 values.push(format!("{}__{}={}", VARIABLE_PREFIX, "call", fn_name));
             }
         }
-        values.join("\n").trim().to_string()
+        values.join("\n")
     }
     fn add_arg(&mut self, arg_data: &'a ArgData, position: &Position) -> Result<()> {
         let arg_data = WrapArgData::new(arg_data, self.positional_idx);
@@ -232,6 +237,7 @@ impl<'a> Cmd<'a> {
 #[derive(Debug)]
 struct WrapArgData<'a> {
     data: &'a ArgData<'a>,
+    arg_name: String,
     value_name: String,
     pos_index: usize,
 }
@@ -247,10 +253,12 @@ impl<'a> WrapArgData<'a> {
     fn new(data: &'a ArgData<'a>, pos_index: usize) -> Self {
         let value_name = data
             .value_name
-            .map(|v| v.to_owned())
-            .unwrap_or_else(|| data.name.to_case(Case::Cobol));
+            .map(to_cobol_case)
+            .unwrap_or_else(|| to_cobol_case(data.name));
+        let arg_name = to_snake_case(data.name);
         Self {
             data,
+            arg_name,
             value_name,
             pos_index,
         }
@@ -308,28 +316,36 @@ impl<'a> WrapArgData<'a> {
                 if self.multiple {
                     arg = arg.multiple_values(true)
                 }
+                if let Some(default) = self.default {
+                    arg = arg.default_value(default);
+                }
                 arg
             }
         };
         Ok(arg)
     }
     fn retrieve_match_value(&self, matches: &ArgMatches) -> Option<String> {
-        let name = self.name.to_case(Case::Snake);
+        let arg_name = self.arg_name.as_str();
         if !matches.is_present(self.name) {
             return None;
         }
         if self.kind == ArgKind::Flag {
-            return Some(format!("{}_{}=1", VARIABLE_PREFIX, name));
+            return Some(format!("{}_{}=1", VARIABLE_PREFIX, arg_name));
         }
         if self.multiple {
             return matches.values_of(self.name).map(|values| {
-                let values: Vec<String> = values.map(escape_value).collect();
-                format!("{}_{}=( {} )", VARIABLE_PREFIX, name, values.join(" "))
+                let values: Vec<String> = values.map(escape_shell_words).collect();
+                format!("{}_{}=( {} )", VARIABLE_PREFIX, arg_name, values.join(" "))
             });
         }
-        matches
-            .value_of(self.name)
-            .map(|value| format!("{}_{}={}", VARIABLE_PREFIX, name, escape_value(value)))
+        matches.value_of(self.name).map(|value| {
+            format!(
+                "{}_{}={}",
+                VARIABLE_PREFIX,
+                arg_name,
+                escape_shell_words(value)
+            )
+        })
     }
     fn detect_conflict(
         &self,
@@ -380,26 +396,3 @@ impl<'a> WrapArgData<'a> {
         Ok(())
     }
 }
-
-fn escape_value(value: &str) -> String {
-    let mut output = String::new();
-    if value.is_empty() {
-        return "''".to_string();
-    }
-    for ch in value.chars() {
-        if !ch.is_ascii() {
-            output.push(ch)
-        }
-        match ch {
-            'A'..='Z' | 'a'..='z' | '0'..='9' | '_' | '-' | '.' | ',' | ':' | '/' | '@' => {
-                output.push(ch)
-            }
-            '\n' => output.push_str("'\n'"),
-            _ => {
-                output.push('\\');
-                output.push(ch);
-            }
-        }
-    }
-    output
-}
