@@ -1,8 +1,8 @@
+mod cli;
 mod parser;
-mod runner;
+mod utils;
 
-use anyhow::bail;
 use anyhow::Error;
-pub use runner::run;
+pub use cli::{run, Runner};
 
 pub type Result<T> = std::result::Result<T, Error>;
